@@ -5,7 +5,7 @@
  * object level logging, and fully-qualified object names for all DML and DDL
  * statements where possible (See README.md for details).
  *
- * Copyright (c) 2014-2022, PostgreSQL Global Development Group
+ * Copyright (c) 2014-2020, PostgreSQL Global Development Group
  *------------------------------------------------------------------------------
  */
 #include "postgres.h"
@@ -19,7 +19,6 @@
 #include "catalog/pg_class.h"
 #include "catalog/namespace.h"
 #include "commands/dbcommands.h"
-#include "commands/extension.h"
 #include "catalog/pg_proc.h"
 #include "commands/event_trigger.h"
 #include "executor/executor.h"
@@ -164,24 +163,6 @@ char *auditRole = NULL;
 #define AUDIT_TYPE_OBJECT   "OBJECT"
 #define AUDIT_TYPE_SESSION  "SESSION"
 
-/*
- * Command, used for SELECT/DML and function calls.
- *
- * We hook into the executor, but we do not have access to the parsetree there.
- * Therefore we can't simply call CreateCommandTag() to get the command and have
- * to build it ourselves based on what information we do have.
- *
- * These should be updated if new commands are added to what the exectuor
- * currently handles.  Note that most of the interesting commands do not go
- * through the executor but rather ProcessUtility, where we have the parsetree.
- */
-#define COMMAND_SELECT      "SELECT"
-#define COMMAND_INSERT      "INSERT"
-#define COMMAND_UPDATE      "UPDATE"
-#define COMMAND_DELETE      "DELETE"
-#define COMMAND_EXECUTE     "EXECUTE"
-#define COMMAND_UNKNOWN     "UNKNOWN"
-
 /*
  * Object type, used for SELECT/DML statements and function calls.
  *
@@ -206,17 +187,6 @@ char *auditRole = NULL;
 
 #define OBJECT_TYPE_UNKNOWN         "UNKNOWN"
 
-/*
- * String constants for testing role commands.  Rename and drop role statements
- * are assigned the nodeTag T_RenameStmt and T_DropStmt respectively.  This is
- * not very useful for classification, so we resort to comparing strings
- * against the result of CreateCommandTag(parsetree).
- */
-#define COMMAND_ALTER_ROLE          "ALTER ROLE"
-#define COMMAND_DROP_ROLE           "DROP ROLE"
-#define COMMAND_GRANT               "GRANT"
-#define COMMAND_REVOKE              "REVOKE"
-
 /*
  * String constants used for redacting text after the password token in
  * CREATE/ALTER ROLE commands.
@@ -237,7 +207,7 @@ typedef struct
     LogStmtLevel logStmtLevel;  /* From GetCommandLogLevel when possible,
                                    generated when not. */
     NodeTag commandTag;         /* same here */
-    const char *command;        /* same here */
+    int command;                /* same here */
     const char *objectType;     /* From event trigger when possible,
                                    generated when not. */
     char *objectName;           /* Fully qualified object identification */
@@ -484,19 +454,6 @@ log_audit_event(AuditEventStackItem *stackItem)
     MemoryContext contextOld;
     StringInfoData auditStr;
 
-    /*
-     * Skip logging script statements if an extension is currently being created
-     * or altered. PostgreSQL reports the statement text for each statement in
-     * the script as the entire script text, which can blow up the logs. The
-     * create/alter statement will still be logged.
-     *
-     * Since a superuser is responsible for determining which extensions are
-     * available, and in most cases installing them, it should not be necessary
-     * to log each statement in the script.
-     */
-    if (creating_extension)
-        return;
-
     /* If this event has already been logged don't log it again */
     if (stackItem->auditEvent.logged)
         return;
@@ -577,6 +534,8 @@ log_audit_event(AuditEventStackItem *stackItem)
                         }
                     }
 
+                /* Fall through */
+
                 /* Classify role statements */
                 case T_GrantStmt:
                 case T_GrantRoleStmt:
@@ -594,10 +553,8 @@ log_audit_event(AuditEventStackItem *stackItem)
                  */
                 case T_RenameStmt:
                 case T_DropStmt:
-                    if (pg_strcasecmp(stackItem->auditEvent.command,
-                                      COMMAND_ALTER_ROLE) == 0 ||
-                        pg_strcasecmp(stackItem->auditEvent.command,
-                                      COMMAND_DROP_ROLE) == 0)
+                    if (stackItem->auditEvent.command == CMDTAG_ALTER_ROLE ||
+                        stackItem->auditEvent.command == CMDTAG_DROP_ROLE)
                     {
                         className = CLASS_ROLE;
                         class = LOG_ROLE;
@@ -685,7 +642,7 @@ log_audit_event(AuditEventStackItem *stackItem)
      * this string is everything else.
      */
     initStringInfo(&auditStr);
-    append_valid_csv(&auditStr, stackItem->auditEvent.command);
+    append_valid_csv(&auditStr, GetCommandTagName(stackItem->auditEvent.command));
 
     appendStringInfoCharMacro(&auditStr, ',');
     append_valid_csv(&auditStr, stackItem->auditEvent.objectType);
@@ -1008,12 +965,19 @@ log_select_dml(Oid auditOid, List *rangeTabls)
         /*
          * Don't log if the session user is not a member of the current
          * role.  This prevents contents of security definer functions
-         * from being logged and supresses foreign key queries unless the
-         * session user is the owner of the referenced table.
+         * from being logged.
          */
         if (!is_member_of_role(GetSessionUserId(), GetUserId()))
             return;
 
+        /*
+         * Don't log if this is not a true update UPDATE command, e.g. a
+         * SELECT FOR UPDATE used for foreign key lookups.
+         */
+        if (rte->requiredPerms & ACL_UPDATE &&
+            rte->rellockmode < RowExclusiveLock)
+            continue;
+
         /*
          * If we are not logging all-catalog queries (auditLogCatalog is
          * false) then filter out any system relations here.
@@ -1050,31 +1014,31 @@ log_select_dml(Oid auditOid, List *rangeTabls)
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_MOD;
             auditEventStack->auditEvent.commandTag = T_InsertStmt;
-            auditEventStack->auditEvent.command = COMMAND_INSERT;
+            auditEventStack->auditEvent.command = CMDTAG_INSERT;
         }
         else if (rte->requiredPerms & ACL_UPDATE)
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_MOD;
             auditEventStack->auditEvent.commandTag = T_UpdateStmt;
-            auditEventStack->auditEvent.command = COMMAND_UPDATE;
+            auditEventStack->auditEvent.command = CMDTAG_UPDATE;
         }
         else if (rte->requiredPerms & ACL_DELETE)
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_MOD;
             auditEventStack->auditEvent.commandTag = T_DeleteStmt;
-            auditEventStack->auditEvent.command = COMMAND_DELETE;
+            auditEventStack->auditEvent.command = CMDTAG_DELETE;
         }
         else if (rte->requiredPerms & ACL_SELECT)
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_ALL;
             auditEventStack->auditEvent.commandTag = T_SelectStmt;
-            auditEventStack->auditEvent.command = COMMAND_SELECT;
+            auditEventStack->auditEvent.command = CMDTAG_SELECT;
         }
         else
         {
             auditEventStack->auditEvent.logStmtLevel = LOGSTMT_ALL;
             auditEventStack->auditEvent.commandTag = T_Invalid;
-            auditEventStack->auditEvent.command = COMMAND_UNKNOWN;
+            auditEventStack->auditEvent.command = CMDTAG_UNKNOWN;
         }
 
         /* Use the relation type to assign object type */
@@ -1248,7 +1212,7 @@ log_function_execute(Oid objectId)
     /* Log the function call */
     stackItem->auditEvent.logStmtLevel = LOGSTMT_ALL;
     stackItem->auditEvent.commandTag = T_DoStmt;
-    stackItem->auditEvent.command = COMMAND_EXECUTE;
+    stackItem->auditEvent.command = CMDTAG_EXECUTE;
     stackItem->auditEvent.objectType = OBJECT_TYPE_FUNCTION;
     stackItem->auditEvent.commandText = stackItem->next->auditEvent.commandText;
 
@@ -1287,31 +1251,31 @@ pgaudit_ExecutorStart_hook(QueryDesc *queryDesc, int eflags)
             case CMD_SELECT:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_ALL;
                 stackItem->auditEvent.commandTag = T_SelectStmt;
-                stackItem->auditEvent.command = COMMAND_SELECT;
+                stackItem->auditEvent.command = CMDTAG_SELECT;
                 break;
 
             case CMD_INSERT:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_MOD;
                 stackItem->auditEvent.commandTag = T_InsertStmt;
-                stackItem->auditEvent.command = COMMAND_INSERT;
+                stackItem->auditEvent.command = CMDTAG_INSERT;
                 break;
 
             case CMD_UPDATE:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_MOD;
                 stackItem->auditEvent.commandTag = T_UpdateStmt;
-                stackItem->auditEvent.command = COMMAND_UPDATE;
+                stackItem->auditEvent.command = CMDTAG_UPDATE;
                 break;
 
             case CMD_DELETE:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_MOD;
                 stackItem->auditEvent.commandTag = T_DeleteStmt;
-                stackItem->auditEvent.command = COMMAND_DELETE;
+                stackItem->auditEvent.command = CMDTAG_DELETE;
                 break;
 
             default:
                 stackItem->auditEvent.logStmtLevel = LOGSTMT_ALL;
                 stackItem->auditEvent.commandTag = T_Invalid;
-                stackItem->auditEvent.command = COMMAND_UNKNOWN;
+                stackItem->auditEvent.command = CMDTAG_UNKNOWN;
                 break;
         }
 
@@ -1372,7 +1336,7 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
                             ParamListInfo params,
                             QueryEnvironment *queryEnv,
                             DestReceiver *dest,
-                            char *completionTag)
+                            QueryCompletion *qc)
 {
     AuditEventStackItem *stackItem = NULL;
     int64 stackId = 0;
@@ -1429,18 +1393,6 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
             !IsAbortedTransactionBlockState())
             log_audit_event(stackItem);
 
-        /*
-         * If this is a create/alter extension command log it before calling
-         * the next ProcessUtility hook. Otherwise, any warnings will be emitted
-         * before the create/alter is logged and errors will prevent it from
-         * being logged at all.
-         */
-        if (auditLogBitmap & LOG_DDL &&
-            (stackItem->auditEvent.commandTag == T_CreateExtensionStmt ||
-                stackItem->auditEvent.commandTag == T_AlterExtensionStmt) &&
-            !IsAbortedTransactionBlockState())
-            log_audit_event(stackItem);
-
         /*
          * A close will free the open cursor which will also free the close
          * audit entry. Immediately log the close and set stackItem to NULL so
@@ -1458,10 +1410,10 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
     /* Call the standard process utility chain. */
     if (next_ProcessUtility_hook)
         (*next_ProcessUtility_hook) (pstmt, queryString, context, params,
-                                     queryEnv, dest, completionTag);
+                                     queryEnv, dest, qc);
     else
         standard_ProcessUtility(pstmt, queryString, context, params,
-                                queryEnv, dest, completionTag);
+                                queryEnv, dest, qc);
 
     /*
      * Process the audit event if there is one.  Also check that this event
@@ -1560,9 +1512,7 @@ pgaudit_ddl_command_end(PG_FUNCTION_ARGS)
         CreateCommandTag(eventData->parsetree);
 
     /* Return objects affected by the (non drop) DDL statement */
-    query = "SELECT pg_catalog.upper(object_type),\n"
-            "       object_identity,\n"
-            "       pg_catalog.upper(command_tag)\n"
+    query = "SELECT UPPER(object_type), object_identity, UPPER(command_tag)\n"
             "  FROM pg_catalog.pg_event_trigger_ddl_commands()";
 
     /* Attempt to connect */
@@ -1591,7 +1541,7 @@ pgaudit_ddl_command_end(PG_FUNCTION_ARGS)
         auditEventStack->auditEvent.objectName =
             SPI_getvalue(spiTuple, spiTupDesc, 2);
         auditEventStack->auditEvent.command =
-            SPI_getvalue(spiTuple, spiTupDesc, 3);
+            GetCommandTagEnum(SPI_getvalue(spiTuple, spiTupDesc, 3));
 
         auditEventStack->auditEvent.logged = false;
 
@@ -1599,8 +1549,8 @@ pgaudit_ddl_command_end(PG_FUNCTION_ARGS)
          * Identify grant/revoke commands - these are the only non-DDL class
          * commands that should be coming through the event triggers.
          */
-        if (pg_strcasecmp(auditEventStack->auditEvent.command, COMMAND_GRANT) == 0 ||
-            pg_strcasecmp(auditEventStack->auditEvent.command, COMMAND_REVOKE) == 0)
+        if (auditEventStack->auditEvent.command == CMDTAG_GRANT ||
+            auditEventStack->auditEvent.command == CMDTAG_REVOKE)
         {
             NodeTag currentCommandTag = auditEventStack->auditEvent.commandTag;
 
@@ -1661,11 +1611,11 @@ pgaudit_sql_drop(PG_FUNCTION_ARGS)
     contextOld = MemoryContextSwitchTo(contextQuery);
 
     /* Return objects affected by the drop statement */
-    query = "SELECT pg_catalog.upper(object_type),\n"
-            "       object_identity\n"
-            "  FROM pg_catalog.pg_event_trigger_dropped_objects()\n"
-            " WHERE pg_catalog.lower(object_type) operator(pg_catalog.<>) 'type'\n"
-            "   AND schema_name operator(pg_catalog.<>) 'pg_toast'";
+    query = "SELECT UPPER(object_type),\n"
+        "       object_identity\n"
+        "  FROM pg_catalog.pg_event_trigger_dropped_objects()\n"
+        " WHERE lower(object_type) <> 'type'\n"
+        "   AND schema_name <> 'pg_toast'";
 
     /* Attempt to connect */
     result = SPI_connect();
