@@ -765,7 +765,7 @@ CREATE TRIGGER bbb_insert_trg
 	AFTER INSERT ON bbb
 	FOR EACH ROW EXECUTE PROCEDURE bbb_insert();
 
-GRANT SELECT
+GRANT SELECT, UPDATE
    ON aaa
    TO auditor;
 
@@ -848,49 +848,6 @@ CREATE INDEX h_idx ON h (x);
 DROP INDEX h_idx;
 DROP TABLE h;
 
---
--- Test that pgaudit event triggers are immune to search-path-based attacks
-
--- Attempt to capture unqualified references to standard functions
-CREATE FUNCTION upper(text) RETURNS text
-LANGUAGE SQL AS 'SELECT (1/0)::text';
-
-CREATE FUNCTION lower(text) RETURNS text
-LANGUAGE SQL AS 'SELECT (1/0)::text';
-
-CREATE FUNCTION my_ne(text, text) RETURNS bool
-LANGUAGE SQL AS 'SELECT (1/0)::bool';
-
-CREATE OPERATOR <> (FUNCTION = my_ne, LEFTARG = text, RIGHTARG = text);
-
-CREATE EXTENSION IF NOT EXISTS pgaudit;
-SET pgaudit.log = 'DDL';
--- Put public schema before pg_catalog to capture unqualified references
-SET search_path = public, pg_catalog;
-
--- If there was a vulnerability, these would fail with division by zero error
-CREATE TABLE wombat ();
-DROP TABLE wombat;
-
-SET pgaudit.log = 'NONE';
-DROP EXTENSION pgaudit;
-
-DROP OPERATOR <> (text, text);
-DROP FUNCTION my_ne(text, text);
-DROP FUNCTION lower(text);
-DROP FUNCTION upper(text);
-
--- Create/drop extension. Note that the log level here must be warning because the create extension code will reset any higher log
--- levels like notice
-SET pgaudit.log = 'all,-misc_set';
-SET pgaudit.log_level = 'warning';
-
-CREATE EXTENSION pg_stat_statements;
-ALTER EXTENSION pg_stat_statements UPDATE TO '1.7';
-DROP EXTENSION pg_stat_statements;
-
-SET pgaudit.log_level = 'notice';
-
 -- Cleanup
 -- Set client_min_messages up to warning to avoid noise
 SET client_min_messages = 'warning';
